﻿

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif
#include <wx/ffile.h>
#include <vector>
#include <algorithm>
//#define _USE_MATH_DEFINES 1
#include <cmath>
#include <stdio.h>

#ifndef M_1_PI
#define M_1_PI 0.318309886183790671538
#endif

#include <gnuradio/top_block.h>
#include <gnuradio/analog/quadrature_demod_cf.h>
#include <gnuradio/analog/probe_avg_mag_sqrd_c.h>
#include <gnuradio/filter/freq_xlating_fir_filter_ccf.h>
#include <gnuradio/filter/firdes.h>
#include <gnuradio/filter/fir_filter_fff.h>
#include <gnuradio/blocks/float_to_complex.h>
#include <gnuradio/blocks/float_to_uchar.h>
#include <gnuradio/blocks/wavfile_sink.h>
#include <gnuradio/blocks/abs_ff.h>
#include <gnuradio/blocks/add_ff.h>
#include <gnuradio/blocks/add_const_ff.h>
#include <gnuradio/blocks/sub_ff.h>
#include <gnuradio/blocks/delay.h>
#include <osmosdr/source.h>

#include "sennh_bat_sink_b.h"
#include "gr_optfir.h"


#include "battery_widget.h"

#ifndef wxHAS_IMAGES_IN_RESOURCES
    #include "batmon16.xpm"
	#include "batmon32.xpm"
	#include "batmon48.xpm"
	#define wxICONS(X,S) wxIcon(X##S##_xpm)
#else
	#define wxICONS(X,S) wxIcon(wxT(#X),wxICON_DEFAULT_TYPE,S,S)
#endif

void ParseCSV(const wxString& csvSource, std::vector<std::vector<wxString> >& lines);

class BatMonApp: public wxApp
{
	public:
	virtual bool OnInit();
};

class BatterySizer: public wxBoxSizer
{
public:
	BatterySizer(wxWindow *parent, const wxString& t, const wxString& n, unsigned int f, const wxColour& bgcol, int s);
	void SetSteps(int s);
	void SetState(int s, int n);
	void SetState(int s);
	void SetType(const wxString& n);
	void SetName(const wxString& n);
	void SetColour(const wxColour& col);
	void IncTimer();
	void Enable(bool e);
	unsigned int GetFreq();
	void DecodeAndSetState(unsigned int s);
	void SetLevel(double level);
private:
	unsigned int freq;
	wxLongLong last_update;
	int steps;
	bool enabled;
	wxStaticText *type_label;
	wxStaticText *name_label;
	wxStaticText *freq_label;
	wxStaticText *update_label;
	wxStaticText *level_label;
	BatteryWidget *batpanel;
};


class BatMonDecoder
{
public:
	BatMonDecoder(gr::top_block_sptr gr_tb, osmosdr::source::sptr source, std::vector<float> gr_lowpass, std::vector<float> gr_audiotaps, std::vector<float> gr_bandtaps, unsigned int gr_xdecim, unsigned int gr_fmdecim, unsigned int gr_sourcerate);
	void SetCallback(void (*cbFunc)(void*, unsigned int), void *cbParam);
	void SetFreqOffset(int f);
	double GetLevel();
	void ResetLevel();
	bool IsMute();
	bool IsEnabled();
	bool GetTelegram(unsigned int *state);
private:
	gr::filter::freq_xlating_fir_filter_ccf::sptr gr_xlate;
	gr::analog::quadrature_demod_cf::sptr gr_quaddemod;
	gr::analog::probe_avg_mag_sqrd_c::sptr gr_level;
	gr::filter::fir_filter_fff::sptr gr_lpffir;
	gr::blocks::sennh_bat_sink_b::sptr gr_senn;
};

class BatMonFrame: public wxFrame
{
public:
	BatMonFrame(const wxString& title, const wxPoint& pos, const wxSize& size);
	void OnTimer(wxTimerEvent& event);
	void OnLevelTimer(wxTimerEvent& event);
	static bool compare_batsizers(BatterySizer *a, BatterySizer *b);
	wxScrolledWindow *batwin;
private:
	wxIconBundle icons;
	wxTimer m_timer;
	wxTimer m_ltimer;
	wxTimerEvent m_timerevent;
	void OnLoadTrans(wxCommandEvent& event);
	void OnExit(wxCommandEvent& event);
	void OnAbout(wxCommandEvent& event);
	void OnSettings(wxCommandEvent& event);
	void InitReceiver();
	void StartReceiver();
	void StopReceiver();
	std::vector <BatterySizer*> bat_blocks;
	int freq_pos;
	int freq_pos_cur;
	int freq_len;
	unsigned int gr_fmdecim, gr_xdecim, gr_range_border;
	unsigned int gr_sourcerate;
	double freq_cor;
	std::vector<BatMonDecoder*> bat_decoders;
	std::vector<float> gr_lowpass;
	std::vector<float> gr_audiotaps;
	std::vector<float> gr_bandtaps;
	gr::top_block_sptr gr_tb;
	osmosdr::source::sptr osmo_source;

	wxDECLARE_EVENT_TABLE();
};


enum
{
	ID_LoadTrans = 1,
	ID_Timer = 2,
	ID_Level_Timer = 3
};

wxBEGIN_EVENT_TABLE(BatMonFrame, wxFrame)
	EVT_MENU(ID_LoadTrans, BatMonFrame::OnLoadTrans)
	EVT_MENU(wxID_PREFERENCES, BatMonFrame::OnSettings)
	EVT_MENU(wxID_EXIT, BatMonFrame::OnExit)
	EVT_MENU(wxID_ABOUT, BatMonFrame::OnAbout)
	EVT_TIMER(ID_Timer, BatMonFrame::OnTimer)
	EVT_TIMER(ID_Level_Timer, BatMonFrame::OnLevelTimer)
wxEND_EVENT_TABLE()


wxIMPLEMENT_APP(BatMonApp);

BatMonDecoder::BatMonDecoder(gr::top_block_sptr gr_tb, osmosdr::source::sptr source, std::vector<float> gr_lowpass, std::vector<float> gr_audiotaps, std::vector<float> gr_bandtaps, unsigned int gr_xdecim, unsigned int gr_fmdecim, unsigned int gr_sourcerate)
{
	//const int xdecim=8, fmdecim=8;
	//const double rate = 3.2e6;
	double k;

	gr_xlate = gr::filter::freq_xlating_fir_filter_ccf::make(gr_xdecim, gr_lowpass, 100000.0, ((double)gr_sourcerate)*1000.0);

    k=(400000.0/((112000.0)*M_PI));
    gr_quaddemod = gr::analog::quadrature_demod_cf::make(k);
    
	gr_level = gr::analog::probe_avg_mag_sqrd_c::make( -25.0, 1.0 / 400000.0);
    gr_lpffir = gr::filter::fir_filter_fff::make(gr_fmdecim, gr_audiotaps);
    
    gr::filter::fir_filter_fff::sptr bandpass = gr::filter::fir_filter_fff::make(1000,gr_bandtaps);
    gr::blocks::add_const_ff::sptr addc = gr::blocks::add_const_ff::make(-0.02);
    gr::blocks::abs_ff::sptr abs = gr::blocks::abs_ff::make();
    gr::blocks::add_ff::sptr add = gr::blocks::add_ff::make(1);
    gr::blocks::delay::sptr delay = gr::blocks::delay::make(sizeof(float),1);
    gr::blocks::sub_ff::sptr sub = gr::blocks::sub_ff::make(1);
	gr::blocks::float_to_complex::sptr fl2cp = gr::blocks::float_to_complex::make(1);
    gr::analog::quadrature_demod_cf::sptr batdemod = gr::analog::quadrature_demod_cf::make(M_1_PI);
	gr::blocks::float_to_uchar::sptr fl2ch = gr::blocks::float_to_uchar::make();

	gr_senn = gr::blocks::sennh_bat_sink_b::make(50);


    gr_tb->connect(source, 0, gr_xlate, 0);
    gr_tb->connect(gr_xlate, 0, gr_quaddemod, 0);
	gr_tb->connect(gr_xlate, 0, gr_level, 0);
    gr_tb->connect(gr_quaddemod, 0, gr_lpffir, 0);
    gr_tb->connect(gr_lpffir, 0, bandpass, 0);
    gr_tb->connect(bandpass, 0, addc, 0);
    gr_tb->connect(addc, 0, add, 0);
    gr_tb->connect(addc, 0, abs, 0);
    gr_tb->connect(abs, 0, add, 1);
    gr_tb->connect(add, 0, delay, 0);
    gr_tb->connect(add, 0, sub, 0);
    gr_tb->connect(delay, 0, sub, 1);
    gr_tb->connect(sub, 0, fl2cp, 0);
    gr_tb->connect(fl2cp, 0, batdemod, 0);
    gr_tb->connect(batdemod, 0, fl2ch, 0);
    gr_tb->connect(fl2ch, 0, gr_senn, 0);
}

void BatMonDecoder::SetFreqOffset(int f)
{
//	wxMessageBox(wxString::Format(wxT("%i"),f),_T("Battery Monitor"),wxOK | wxICON_ERROR);
	gr_xlate->set_center_freq((double)(f*1000));
	gr_senn->reset();
}

void BatMonDecoder::SetCallback(void (*cbFunc)(void*, unsigned int), void *cbParam)
{
	//gr_senn->setcallback(cbFunc, cbParam);
}

double BatMonDecoder::GetLevel()
{
	return gr_level->level();
}

bool BatMonDecoder::GetTelegram(unsigned int *state)
{
	return gr_senn->gettelegram(state);
}

bool BatMonDecoder::IsMute()
{
	return std::log10(gr_level->level())*10<(-30);
}

bool BatMonDecoder::IsEnabled()
{
	return std::log10(gr_level->level())*10>(-18);
}

void BatMonDecoder::ResetLevel()
{
	//gr_level->reset();
}

BatterySizer::BatterySizer(wxWindow *parent, const wxString& t, const wxString& n, unsigned int f, const wxColour& bgcol, int s)
: wxBoxSizer( wxVERTICAL )
{
	update_label = new wxStaticText(parent, wxID_ANY, _T("-"), wxDefaultPosition, wxSize(120,24), wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE  );
	level_label = new wxStaticText(parent, wxID_ANY, _T("-"), wxDefaultPosition, wxSize(120,24), wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE  );
	type_label = new wxStaticText(parent, wxID_ANY, t, wxDefaultPosition, wxSize(120,24), wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE  );
	name_label = new wxStaticText(parent, wxID_ANY, n, wxDefaultPosition, wxSize(120,24), wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE );
	wxString fs;
	freq=f;
	if(f < 1000) fs = wxString::Format(wxT("%03i"),f);
	else if (f < 1000000) fs = wxString::Format(wxT("%i.%03i"),f/1000,f%1000);
	else fs = wxString::Format(wxT("%i%03i.%03i"),f/1000000,(f%1000000)/1000,f%1000);
	freq_label = new wxStaticText(parent, wxID_ANY, fs, wxDefaultPosition, wxSize(120,24), wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE );
	batpanel = new BatteryWidget(parent, wxID_ANY, wxDefaultPosition, wxSize(120,320));
	wxFont font = wxFont(16, wxDEFAULT, wxNORMAL, wxBOLD);
	last_update = 0;
	enabled = true;
	steps = s;
	type_label->SetFont(font);
	type_label->SetForegroundColour(wxColour(0xdddddd));
	name_label->SetFont(font);
	name_label->SetForegroundColour(wxColour(0xdddddd));
	freq_label->SetFont(font);
	freq_label->SetForegroundColour(wxColour(0xdddddd));
	update_label->SetFont(font);
	update_label->SetForegroundColour(wxColour(0xdddddd));
	level_label->SetFont(font);
	level_label->SetForegroundColour(wxColour(0xdddddd));
	batpanel->SetSteps(s);
	batpanel->SetBackgroundColour(bgcol);
	batpanel->SetBorder(6);
	Add(update_label);
	Add(level_label);
	Add(batpanel);
	Add(type_label);
	Add(freq_label);
	Add(name_label);
}

void BatterySizer::SetColour(const wxColour& col)
{
	batpanel->SetColour(col);
}

unsigned int BatterySizer::GetFreq()
{
	return freq;
}


void BatterySizer::Enable(bool e)
{
	if(e==enabled) return;
	enabled = e;
	
	if(e) {
		SetState(1,1);
	}
	else {
		batpanel->SetState(0, 1);
		batpanel->SetColour(wxColour(0x777777));
		update_label->SetLabelText("");
	}

}

void BatterySizer::SetState(int s)
{
	SetState(s, batpanel->GetSteps());
}

void BatterySizer::SetState(int s, int n)
{
	int x;
	steps = n;
	if(!enabled) Enable(true);
	if(n>1) {
		x=n+1;
		if (s>=(x>>1)) SetColour(wxColour(0x00dd00));
		else if (s>(x>>2)) SetColour(wxColour(0x00aaff));
		else SetColour(wxColour(0x0000ee));
	}
	else {
		SetColour(wxColour(0xdddddd));
	}
	batpanel->SetState(s, n);
	last_update =  wxGetUTCTime();
	update_label->SetLabelText(_T("0 sec"));
}

void BatterySizer::DecodeAndSetState(unsigned int s)
{
	switch(s)
	{
	case 0x200:
		SetState(1,3);
		break;
	case 0x300:
		SetState(2,3);
		break;
	case 0x380:
		SetState(3,3);
		break;
	case 0x3c0:
		SetState(0,5);
		break;
	case 0x3e0:
		SetState(1,5);
		break;
	case 0x3f0:
		SetState(2,5);
		break;
	case 0x3f8:
		SetState(3,5);
		break;
	case 0x3fc:
		SetState(4,5);
		break;
	case 0x3fe:
		SetState(5,5);
		break;
	}
}

void BatterySizer::IncTimer()
{
	if(!enabled) return;
	wxLongLong r = wxGetUTCTime()-last_update;
	update_label->SetLabelText(wxString::Format(wxT("%i sec"),(int)r.GetLo()));
}

void BatterySizer::SetLevel(double level)
{
	level_label->SetLabelText(wxString::Format(wxT("%.2f dB"),(std::log10(level)*10.0)));
}

bool BatMonApp::OnInit()
{
	BatMonFrame *frame = new BatMonFrame( _T("Battery Monitor"), wxPoint(50, 50), wxSize(720, 600) );
	frame->Show( true );
	return true;
}

bool BatMonFrame::compare_batsizers(BatterySizer *a, BatterySizer *b)
{
	return (a->GetFreq() < b->GetFreq());
}

BatMonFrame::BatMonFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
	: wxFrame(NULL, wxID_ANY, title, pos, size), m_timer(this, ID_Timer), m_ltimer(this, ID_Level_Timer)
{
	icons.AddIcon(wxICONS(batmon,16));
	icons.AddIcon(wxICONS(batmon,32));
	icons.AddIcon(wxICONS(batmon,48));
	SetBackgroundColour(wxColour(0x333333));
	SetIcons(icons);
	wxMenu *menuFile = new wxMenu;
	menuFile->Append(ID_LoadTrans, _T("&Load transmitter configuration...\tCtrl-L"),
	_T("Load transmitter names, types and frequencies from a CSV file"));
	menuFile->AppendSeparator();
	menuFile->Append(wxID_PREFERENCES, _T("&Settings...\tCtrl-S"),
	_T("Show program settings"));
	menuFile->AppendSeparator();
	menuFile->Append(wxID_EXIT);
	wxMenu *menuHelp = new wxMenu;
	menuHelp->Append(wxID_ABOUT);
	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append( menuFile, "&File" );
	menuBar->Append( menuHelp, "&Help" );
	SetMenuBar( menuBar );
	CreateStatusBar();
	SetStatusText( _T("Battery Monitor Version 0.0.0") );
	batwin = NULL;
	freq_pos = 0;
	freq_len = 0;
	freq_cor = 1.0001063050519; // <- empirical value for my rtl-sdr, 1=no correction
	gr_sourcerate = 2400;

}

void BatMonFrame::OnExit(wxCommandEvent& event)
{
	Close( true );
}
void BatMonFrame::OnAbout(wxCommandEvent& event)
{
	wxMessageBox( _T("Battery Monitor Version 0.0.0\n © 2014 Stefan Oltmanns\n\nThis software is released under the GNU General Public Licence v3 or at your option any later version.\n\nBattery Monitor is powered by:\n GnuRadio © Free Software Foundation\n wxWidgets © wxWidgets Team\n"),
		_T("About Battery Monitor"), wxOK | wxICON_INFORMATION );
	StopReceiver();
}
void BatMonFrame::OnLoadTrans(wxCommandEvent& event)
{
	//wxLogMessage(_T("Show a open file dialog..."));
	//fprintf(stderr,"alles ok0\n");
	wxFileDialog openFileDialog(this, _T("Open Transmitter configuration..."), "", "", _T("CSV files (*.csv)|*.csv|All files (*.*)|*.*"), wxFD_OPEN|wxFD_FILE_MUST_EXIST);
	if (openFileDialog.ShowModal() == wxID_CANCEL)
		return;
	wxFFile file(openFileDialog.GetPath());

	if(!file.IsOpened()) {
		//wxMessageBox(_T("Could not open file."),_T("Battery Monitor"),wxOK | wxICON_ERROR);
		return;
	}
	wxString file_content;
	file.ReadAll(&file_content,wxConvUTF8);
	file.Close();
	if(file_content.IsEmpty()) {
		wxMessageBox(_T("File is not valid CSV."),_T("Battery Monitor"),wxOK | wxICON_ERROR);
		return;
	}

	std::vector<std::vector<wxString> > csv_content;
	ParseCSV(file_content, csv_content);

	std::vector<BatterySizer> bat;


/// TODO !!!! crash if Gnuradio not started ///
	//if(gr_tb-> StopReceiver();

	if(batwin) batwin->Destroy();

	batwin = new wxScrolledWindow(this,wxID_ANY);
	batwin->SetBackgroundColour(wxColour(0x333333));
	wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );

	bat_blocks.clear();

	wxBoxSizer *bat_sizer = new wxBoxSizer(wxHORIZONTAL);
	bat_sizer->Add(8,0,0);
	bat_sizer->AddStretchSpacer();

	for (int i=0; i<csv_content.size(); i++) {
		BatterySizer *batpanel;
		long freq = 0; 
		if (csv_content[i].size()<3) {
			wxMessageBox(_T("File is not valid CSV."),_T("Battery Monitor"),wxOK | wxICON_ERROR);
			break;
		}
		csv_content[i][2].ToLong(&freq,10);
		if(freq<1000) {
			wxMessageBox(_T("CSV file contained invalid frequency"),_T("Battery Monitor"),wxOK | wxICON_ERROR);
			break;
		}
		batpanel = new BatterySizer( batwin, csv_content[i][1], csv_content[i][0], (unsigned int)freq, wxColour(0x333333), 1 );
		batpanel->Enable(false);
		bat_blocks.push_back(batpanel);
		bat_sizer->Add(batpanel);
		bat_sizer->AddStretchSpacer();
		//bat_sizer->Add(8,0,0);
	}

	if(bat_blocks.size()==0) return;

	
	bat_sizer->SetMinSize(bat_blocks.size()*136,0);

	topsizer->Add(0,16,0);

	topsizer->Add(bat_sizer, 1, wxEXPAND | wxALIGN_CENTER ); // no border and centre horizontally
	topsizer->Add(0,16,0);



	batwin->SetSizerAndFit(topsizer);
	batwin->SetScrollRate(5, 5);

	//SetSizerAndFit(batwin);

	std::sort(bat_blocks.begin(),bat_blocks.end(),&BatMonFrame::compare_batsizers);
	//if(!gr_tb)
	//fprintf(stderr,"alles ok1\n");
	InitReceiver();
	//fprintf(stderr,"alles ok2\n");
	StartReceiver();
	

}
void BatMonFrame::OnSettings(wxCommandEvent& event)
{
	wxMessageBox(_T("Settings are not implemented yet :("),_T("Battery Monitor"),wxOK);
	//bat_blocks[0]->Enable(false);
	//InitReceiver();
	//StartReceiver();
}

void BatMonFrame::InitReceiver()
{

	//const int xdecim=8, fmdecim=8;
	const double rate = ((double)gr_sourcerate) * 1000.0;

	gr_xdecim = gr_sourcerate/400;
	gr_fmdecim = 8;
	

	gr_tb = gr::make_top_block("Battery Signal Receiver");
	osmo_source = osmosdr::source::make( "numchan=1" );
	osmo_source->set_sample_rate(rate);
	osmo_source->set_center_freq(800000000.0, 0);
	osmo_source->set_freq_corr(0, 0);
	osmo_source->set_dc_offset_mode(0, 0);
	osmo_source->set_iq_balance_mode(0, 0);
	osmo_source->set_gain(10, 0);
	osmo_source->set_if_gain(20, 0);
	osmo_source->set_bb_gain(20, 0);
	osmo_source->set_gain_mode(true, 0);
	osmo_source->set_antenna("", 0);
	osmo_source->set_bandwidth(0, 0);

	gr_lowpass = gr::filter::firdes::low_pass(1, rate, 115000, 30000);
	gr_audiotaps = gr_optfir::low_pass(1.0, rate/gr_xdecim, 15000, 16000, 0.1, 60);
	gr_bandtaps = gr::filter::firdes::band_pass(1, rate/(gr_xdecim*gr_fmdecim), 1, 7, 1);

	for(int i=0;i<4;i++) {
		BatMonDecoder *decoder;
		decoder = new BatMonDecoder(gr_tb, osmo_source, gr_lowpass, gr_audiotaps, gr_bandtaps, gr_xdecim, gr_fmdecim, gr_sourcerate);
		bat_decoders.push_back(decoder);
	}
	
}

void BatMonFrame::StartReceiver()
{
	gr_tb->start();
	m_timer.Stop();
	OnTimer(m_timerevent);
	SetStatusText( _T("Receiver started") );
}

void BatMonFrame::StopReceiver()
{
	m_timer.Stop();
	gr_tb->stop();
	SetStatusText( _T("Receiver stopped") );
}

void BatMonFrame::OnLevelTimer(wxTimerEvent& event)
{
	//wxMessageBox(_T("blub"),_T("Battery Monitor"),wxOK | wxICON_ERROR);
	bool ismute = true;
	unsigned int telegram;
	for(int i=0;i<freq_len;i++) {
		bat_blocks[freq_pos_cur+i]->SetLevel(bat_decoders[i]->GetLevel());
		if (bat_decoders[i]->GetTelegram(&telegram)) bat_blocks[freq_pos_cur+i]->DecodeAndSetState(telegram);
		bat_blocks[freq_pos_cur+i]->Enable(bat_decoders[i]->IsEnabled());
		ismute = ismute && bat_decoders[i]->IsMute();
	}
	if(ismute) {
		m_timer.Stop();
		OnTimer(m_timerevent);
	}
}


void BatMonFrame::OnTimer(wxTimerEvent& event)
{
	m_timer.Stop();
	for(int i=0;i<bat_blocks.size();i++) {
		bat_blocks[i]->IncTimer();
	}
	int center_freq = bat_blocks[freq_pos]->GetFreq()+(gr_sourcerate>>1)-400;
	//fprintf(stderr,"Acenter %i pos: %i\n",center_freq, freq_pos);
	int end_freq = center_freq+(gr_sourcerate>>1)-400;
	m_ltimer.Stop();
	for(int i=0;i<bat_decoders.size();i++) bat_decoders[i]->SetCallback(NULL,NULL);
	osmo_source->set_center_freq(((double)(center_freq*1000))*freq_cor);
	SetStatusText(wxString::Format(wxT("Receiver tuned to center frequency: %i kHz"), center_freq ));
	//bat_decoders[0]->SetFreqOffset(((int)bat_blocks[freq_pos]->GetFreq())-center_freq);
	//bat_decoders[0]->SetCallback(&(BatterySizer::DecodeAndSetState), bat_blocks[freq_pos]);
	//fprintf(stderr,"Bcenter %i pos: %i\n",center_freq, freq_pos);
	freq_pos_cur = freq_pos;
	//freq_pos++;
	for(freq_len=0;freq_len<bat_decoders.size() && freq_pos<bat_blocks.size();freq_len++) {
		if(bat_blocks[freq_pos]->GetFreq()<=end_freq)
		{
			bat_decoders[freq_len]->SetFreqOffset((int)(bat_blocks[freq_pos]->GetFreq())-center_freq);
			//bat_decoders[freq_len]->SetCallback(&(BatterySizer::DecodeAndSetState), bat_blocks[freq_pos]);
			freq_pos++;
		}
		else break;
	}
	for(int i=0;i<freq_len;i++) {
		bat_decoders[i]->ResetLevel();
	}
	//gr_tb->stop();
	//gr_tb->start();
	m_ltimer.Start(500);
	if(freq_pos>=bat_blocks.size()) freq_pos = 0;
	m_timer.Start(10000);
}
