#include "sennh_bat_sink_b_impl.h"
#include <gnuradio/io_signature.h>
#include <gnuradio/thread/thread.h>

namespace gr {
  namespace blocks {

    sennh_bat_sink_b::sptr
    sennh_bat_sink_b::make(unsigned int sample_rate)
    {
      return gnuradio::get_initial_sptr
	(new sennh_bat_sink_b_impl(sample_rate));
    }

    sennh_bat_sink_b_impl::sennh_bat_sink_b_impl(unsigned int sample_rate)
      : sync_block("sennh_bat_sink_b",
		      io_signature::make(1, 1, sizeof(unsigned char)),
		      io_signature::make(0, 0, 0)),
	d_sample_rate(sample_rate), d_curtelegram(0), d_lasttelegram(0), d_tpos(-1), d_ncnt(0), d_havetelegram(false), d_cbFunc(0)
    {
		
		d_newblock_cnt = MIN_LEN(NEW_BLOCK_LEN,sample_rate);
	    d_zero_cnt_min = MIN_LEN(ZERO_LEN,sample_rate);
	    d_zero_cnt_max = MAX_LEN(ZERO_LEN,sample_rate);
		d_one_cnt_min  = MIN_LEN(ONE_LEN,sample_rate);
		d_one_cnt_max  = MAX_LEN(ONE_LEN,sample_rate);
		return;
    }
    
    void
    sennh_bat_sink_b_impl::reset()
    {
	  gr::thread::scoped_lock guard(d_mutex);
      d_curtelegram=0;
      d_lasttelegram=0;
      d_tpos=-1;
      d_ncnt=0;
      d_havetelegram=false;
    }
    
    void
    sennh_bat_sink_b_impl::setcallback(void (*cbFunc)(void*, unsigned int), void *cbParam)
    {
		gr::thread::scoped_lock guard(d_mutex);
		d_cbFunc = cbFunc;
		d_cbParam = cbParam;
	}
    
    bool
    sennh_bat_sink_b_impl::gettelegram(unsigned int *telegram)
    {
		gr::thread::scoped_lock guard(d_mutex);
		if(!d_havetelegram) return false;
		*telegram = d_lasttelegram;
		return true;
	}

    sennh_bat_sink_b_impl::~sennh_bat_sink_b_impl()
    {
      stop();
    }

    bool sennh_bat_sink_b_impl::stop()
    {
      return true;
    }

    int
    sennh_bat_sink_b_impl::work(int noutput_items,
			    gr_vector_const_void_star &input_items,
			    gr_vector_void_star &output_items)
    {
      unsigned char **in = (unsigned char**)&input_items[0];

      int nwritten;

      gr::thread::scoped_lock guard(d_mutex);    // hold mutex for duration of this block


	  for(nwritten = 0; nwritten < noutput_items; nwritten++) {
		  if(in[0][nwritten]==0) {
			  d_ncnt++;
			   if (d_tpos==10 && d_ncnt>=d_newblock_cnt) {
				  d_lasttelegram = d_curtelegram;
				  d_havetelegram = true;
				  d_curtelegram=0;
				  d_tpos=-1;
				  if(d_cbFunc) d_cbFunc(d_cbParam, d_lasttelegram);
			   }
		   }
		  else {
			  if(d_tpos<0 && d_ncnt>=d_newblock_cnt) {
			     d_tpos=0;
			  }
			  else if(d_tpos>=0) {
			    if(d_ncnt>= d_zero_cnt_min && d_ncnt<= d_zero_cnt_max && d_tpos < 10) {
				  d_tpos++;
				}
			    else if(d_ncnt>= d_one_cnt_min && d_ncnt<= d_one_cnt_max && d_tpos < 10) {
				  d_curtelegram |= (1<<d_tpos);
				  d_tpos++;
				}
				else { // lost signal
				  d_curtelegram=0;
				  d_tpos=-1;
				}
			  }
			 d_ncnt=0;
		  }
	  }


      return nwritten;
    }



  } /* namespace blocks */
} /* namespace gr */
